/**
* @file    MAIN.cpp
* @author  Thales Vieira <thalesv@gmail.com>
* @author  Instituto de Matemtica, UFAL
* @version 0.1
* @date    24/05/2010
*
* @brief   Lendo e escrevendo arquivos PPM
* @brief   Aula 20
*/
//________________________________________________

#include <windows.h>
#include <iostream>
#include <glut.h>

#include <math.h>

#define PI 3.141592653589793238462643

typedef struct {
	double x;
	double y;
	double z;
} vetor3D;

vetor3D ultimo_mouse;

float *modelview_matrix;


void imprimirMatriz(float *matriz)
{
	for(int i=0;i<4;++i)
	{
		printf("\n");
		for(int j=0;j<4;++j)
			printf("%f ",matriz[i*4+j]);
	}
}

double modulo(vetor3D v)
{
	return sqrt(v.x*v.x+v.y*v.y+v.z*v.z);
}

vetor3D versor(vetor3D v)
{
	vetor3D u;
	double m=modulo(v);
	u.x=v.x/m;
	u.y=v.y/m;
	u.z=v.z/m;
	return u;
}

vetor3D vetorial(vetor3D v1, vetor3D v2)
{
	vetor3D vet;
	vet.x=v1.y*v2.z - v2.y*v1.z;
	vet.y=-v1.x*v2.z + v2.x*v1.z;
	vet.z=v1.x*v2.y - v2.x*v1.y;

	return vet;
}

double escalar(vetor3D v1, vetor3D v2)
{
	return v1.x*v2.x+v1.y*v2.y+v1.z*v2.z;
}


vetor3D projetarNaEsfera(int x, int y)
{
	int largura=glutGet(GLUT_WINDOW_WIDTH);
	int altura=glutGet(GLUT_WINDOW_HEIGHT);

	vetor3D projecao;
	projecao.x=(double)(2.*x-largura)/(double)largura;
	projecao.y=(double)(2.*y-altura)/(double)altura;
	projecao.z=0;

	double comprimento;
	if((comprimento=modulo(projecao))>1.0)
		comprimento=1.0;
	projecao.z=sqrt(1.0-comprimento*comprimento);
	return versor(projecao);
}



void DesenharCubo()
{

	glBegin(GL_QUADS);

	glColor3f(1.0,0.0,0.0);

	glVertex3f(0.5,-0.5,-0.5);
	glVertex3f(0.5,0.5,-0.5);
	glVertex3f(0.5,0.5,0.5);
	glVertex3f(0.5,-0.5,0.5);

	glEnd();
	glBegin(GL_QUADS);


	glVertex3f(-0.5,-0.5,-0.5);
	glVertex3f(-0.5,0.5,-0.5);
	glVertex3f(-0.5,0.5,0.5);
	glVertex3f(-0.5,-0.5,0.5);

	glColor3f(0.0,1.0,0.0);

	glEnd();
	glBegin(GL_QUADS);

	glVertex3f(-0.5,-0.5,-0.5);
	glVertex3f(0.5,-0.5,-0.5);
	glVertex3f(0.5,0.5,-0.5);
	glVertex3f(-0.5,0.5,-0.5);

	glEnd();
	glBegin(GL_QUADS);

	glVertex3f(-0.5,-0.5,0.5);
	glVertex3f(0.5,-0.5,0.5);
	glVertex3f(0.5,0.5,0.5);
	glVertex3f(-0.5,0.5,0.5);

	glColor3f(0.0,0.0,1.0);

	glEnd();
	glBegin(GL_QUADS);

	glVertex3f(-0.5,-0.5,-0.5);
	glVertex3f(0.5,-0.5,-0.5);
	glVertex3f(0.5,-0.5,0.5);
	glVertex3f(-0.5,-0.5,0.5);

	glEnd();
	glBegin(GL_QUADS);

	glVertex3f(-0.5,0.5,-0.5);
	glVertex3f(0.5,0.5,-0.5);
	glVertex3f(0.5,0.5,0.5);
	glVertex3f(-0.5,0.5,0.5);

	glEnd();

	glLineWidth(3);
	glColor3f(1.0,1.0,0.0);

	glBegin(GL_LINE_LOOP);

	glVertex3f(0.5,-0.5,-0.5);
	glVertex3f(0.5,0.5,-0.5);
	glVertex3f(0.5,0.5,0.5);
	glVertex3f(0.5,-0.5,0.5);

	glEnd();

	glBegin(GL_LINE_LOOP);

	glVertex3f(-0.5,-0.5,-0.5);
	glVertex3f(-0.5,0.5,-0.5);
	glVertex3f(-0.5,0.5,0.5);
	glVertex3f(-0.5,-0.5,0.5);

	glEnd();

	glBegin(GL_LINE_LOOP);

	glVertex3f(-0.5,-0.5,-0.5);
	glVertex3f(0.5,-0.5,-0.5);
	glVertex3f(0.5,0.5,-0.5);
	glVertex3f(-0.5,0.5,-0.5);

	glEnd();

	glBegin(GL_LINE_LOOP);

	glVertex3f(-0.5,-0.5,0.5);
	glVertex3f(0.5,-0.5,0.5);
	glVertex3f(0.5,0.5,0.5);
	glVertex3f(-0.5,0.5,0.5);

	glEnd();

	glBegin(GL_LINE_LOOP);

	glVertex3f(-0.5,-0.5,-0.5);
	glVertex3f(0.5,-0.5,-0.5);
	glVertex3f(0.5,-0.5,0.5);
	glVertex3f(-0.5,-0.5,0.5);

	glEnd();

	glBegin(GL_LINE_LOOP);

	glVertex3f(-0.5,0.5,-0.5);
	glVertex3f(0.5,0.5,-0.5);
	glVertex3f(0.5,0.5,0.5);
	glVertex3f(-0.5,0.5,0.5);

	glEnd();

}

void display() {

   glClearColor(0.0, 0.0, 0.0, 0.0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   glPushMatrix();
   glLoadIdentity();

   gluLookAt(0.0,0.0,-4.0,0,0,1.0,0.0,1.0,0.0);

   glMultMatrixf( (GLfloat *) modelview_matrix );

 //  imprimirMatriz(modelview_matrix);


   DesenharCubo();

   glPopMatrix();

   glutSwapBuffers(); //Atualiza o BUFFER
}

/***
* init():
*   Configura o OpenGL.
***/
void init() {
   glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.f,1.0,0.001,100.);
//	glOrtho(-2.0,2.0,-2.0,2.0,-10.0,10.0);
    glMatrixMode(GL_MODELVIEW);
   	glLoadIdentity();
	glGetFloatv(GL_MODELVIEW_MATRIX,(GLfloat *) modelview_matrix);
	glEnable(GL_DEPTH_TEST);

	//	imprimirMatriz(modelview_matrix);
}

void push(int button, int state, int x, int y)
{

    ultimo_mouse=projetarNaEsfera(x,y);
}

void drag(int x, int y)
{
	printf("\n ln %f %f",ultimo_mouse.x,ultimo_mouse.y);


	vetor3D p1=projetarNaEsfera(x,y);
	//printf("\n%f %f %f",p1.x,p1.y,p1.z);
	vetor3D eixo_rot = vetorial(ultimo_mouse,p1);

	vetor3D p1_unit=versor(p1);
	vetor3D eixo_rot_unit=versor(eixo_rot);

	double angulo = acos(escalar(p1_unit,ultimo_mouse));


	glLoadIdentity();
	glRotatef(angulo*180.0/PI,eixo_rot_unit.x,eixo_rot_unit.y,eixo_rot_unit.z);
	glMultMatrixf( (GLfloat *) modelview_matrix );
	glGetFloatv(GL_MODELVIEW_MATRIX,(GLfloat *) modelview_matrix);

	ultimo_mouse.x=p1_unit.x;
	ultimo_mouse.y=p1_unit.y;
	ultimo_mouse.z=p1_unit.z;

	printf(" %f %f",ultimo_mouse.x,ultimo_mouse.y);

	glutPostRedisplay();
}

void writePPM(float *pixels, int width, int height, char *ppmfile)
{
	FILE *arq = fopen( ppmfile, "w" );
	fprintf(arq,"P3\n%d %d\n255\n",width,height);
	for(int h=height-1;h>-1;h--)
	{
		for(int w=0;w<width;w++)
		{
			fprintf(arq,"%d %d %d ",(int)((pixels[3*(h*width+w)])*255),(int)((pixels[3*(h*width+w)+1])*255),
				(int)((pixels[3*(h*width+w)+2])*255));
		}
		fprintf(arq,"\n");
	}
	fclose(arq);
}

void keyboard_callback(unsigned char key, int x, int y) 
{+
	if (key == 'C') 
	{
		int w=glutGet(GLUT_WINDOW_WIDTH);
		int h=glutGet(GLUT_WINDOW_HEIGHT);
		display();
		float *novospixels=new float[h*w*3];
		glReadPixels(0,0,w,h,GL_RGB,GL_FLOAT,novospixels);
		writePPM(novospixels,w,h,"screenshot.ppm");
		delete [] novospixels;
		printf("\nScreenshot salvo no arquivo screenshot.ppm");
	}	
}

/***
* main():
*    Esta  a funo que  chamada quando executamos o programa. Vamos inicializar janelas
e configurar o GLUT aqui.
***/
int main(int argc, char** argv)
{
	modelview_matrix=new float[16];
   // Inicializa o ambiente do GLUT
   glutInit(&argc, argv);

   // Configuraes de janela e de sistema de cor.
   glutInitDisplayMode (GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
   glutInitWindowSize(250,250);
   glutInitWindowPosition(100,100);

   // Cria a janela do GLUT com o ttulo "Hello World!"
   glutCreateWindow("Hello World!");

   // Determina qual funo deve ser chamada para desenhar na tela (draw)
   glutDisplayFunc(display);

   // Determina qual funo deve ser chamada quando o usurio apertar o boto do mouse
   glutMouseFunc (push);
      // Determina qual funo deve ser chamada quando o usurio realiza DRAG
   glutMotionFunc(drag);

   glutKeyboardFunc(keyboard_callback); 

   // Esta funo inicializa configuraes do OpenGL
   init();

   // Exibe a janela e desenha na tela, usando a funo determinada em glutDisplayFunc
   glutMainLoop();
   return 0;
}