/**
* @file    MAIN.cpp
* @author  Thales Vieira <thalesv@gmail.com>
* @author  Instituto de Matemtica, UFAL
* @version 0.1
* @date    24/05/2010
*
* @brief   Lendo e escrevendo arquivos PPM
* @brief   Aula 20
*/
//________________________________________________

#include <windows.h>
#include <iostream>
#include <glut.h>

#include <math.h>

#define PI 3.141592653589793238462643

typedef struct {
	double x;
	double y;
	double z;
} vetor3D;

vetor3D ultimo_mouse;

float *modelview_matrix;

GLuint texture[2];


void imprimirMatriz(float *matriz)
{
	for(int i=0;i<4;++i)
	{
		printf("\n");
		for(int j=0;j<4;++j)
			printf("%f ",matriz[i*4+j]);
	}
}

double modulo(vetor3D v)
{
	return sqrt(v.x*v.x+v.y*v.y+v.z*v.z);
}

vetor3D versor(vetor3D v)
{
	vetor3D u;
	double m=modulo(v);
	u.x=v.x/m;
	u.y=v.y/m;
	u.z=v.z/m;
	return u;
}

vetor3D vetorial(vetor3D v1, vetor3D v2)
{
	vetor3D vet;
	vet.x=v1.y*v2.z - v2.y*v1.z;
	vet.y=-v1.x*v2.z + v2.x*v1.z;
	vet.z=v1.x*v2.y - v2.x*v1.y;

	return vet;
}

double escalar(vetor3D v1, vetor3D v2)
{
	return v1.x*v2.x+v1.y*v2.y+v1.z*v2.z;
}


vetor3D projetarNaEsfera(int x, int y)
{
	int largura=glutGet(GLUT_WINDOW_WIDTH);
	int altura=glutGet(GLUT_WINDOW_HEIGHT);

	vetor3D projecao;
	projecao.x=(double)(2.*x-largura)/(double)largura;
	projecao.y=(double)(2.*y-altura)/(double)altura;
	projecao.z=0;

	double comprimento;
	if((comprimento=modulo(projecao))>1.0)
		comprimento=1.0;
	projecao.z=sqrt(1.0-comprimento*comprimento);
	return versor(projecao);
}


void DesenharCubo()
{
    glEnable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);

	glColor3f(1.0,0.0,0.0);

	glBindTexture(GL_TEXTURE_2D, texture[0]);
	
	glTexCoord2f(0.0,0.0);
	glVertex3f(0.5,-0.5,-0.5);
	glTexCoord2f(1.0,0.0);
	glVertex3f(0.5,0.5,-0.5);
	glTexCoord2f(1.0,1.0);
	glVertex3f(0.5,0.5,0.5);
	glTexCoord2f(0.0,1.0);
	glVertex3f(0.5,-0.5,0.5);

	glEnd();
	glBegin(GL_QUADS);

	glTexCoord2f(0.0,0.0);
	glVertex3f(-0.5,-0.5,-0.5);
	glTexCoord2f(1.0,0.0);
	glVertex3f(-0.5,0.5,-0.5);
	glTexCoord2f(1.0,1.0);
	glVertex3f(-0.5,0.5,0.5);
	glTexCoord2f(0.0,1.0);
	glVertex3f(-0.5,-0.5,0.5);

	glColor3f(0.0,1.0,0.0);

	glEnd();

	glBindTexture(GL_TEXTURE_2D, texture[1]);

	glBegin(GL_QUADS);

	glTexCoord2f(0.0,0.0);
	glVertex3f(-0.5,-0.5,-0.5);
	glTexCoord2f(1.0,0.0);
	glVertex3f(0.5,-0.5,-0.5);
	glTexCoord2f(1.0,1.0);
	glVertex3f(0.5,0.5,-0.5);
	glTexCoord2f(0.0,1.0);
	glVertex3f(-0.5,0.5,-0.5);

	glEnd();
	glBegin(GL_QUADS);

	glTexCoord2f(0.0,0.0);
	glVertex3f(-0.5,-0.5,0.5);
	glTexCoord2f(1.0,0.0);
	glVertex3f(0.5,-0.5,0.5);
	glTexCoord2f(1.0,1.0);
	glVertex3f(0.5,0.5,0.5);
	glTexCoord2f(0.0,1.0);
	glVertex3f(-0.5,0.5,0.5);

	glColor3f(0.0,0.0,1.0);

	glEnd();

	glBindTexture(GL_TEXTURE_2D, texture[0]);
	
	glBegin(GL_QUADS);

	glTexCoord2f(0.0,0.0);
	glVertex3f(-0.5,-0.5,-0.5);
	glTexCoord2f(1.0,0.0);
	glVertex3f(0.5,-0.5,-0.5);
	glTexCoord2f(1.0,1.0);
	glVertex3f(0.5,-0.5,0.5);
	glTexCoord2f(0.0,1.0);
	glVertex3f(-0.5,-0.5,0.5);

	glEnd();
	glBegin(GL_QUADS);

	glTexCoord2f(0.0,0.0);
	glVertex3f(-0.5,0.5,-0.5);
	glTexCoord2f(1.0,0.0);
	glVertex3f(0.5,0.5,-0.5);
	glTexCoord2f(1.0,1.0);
	glVertex3f(0.5,0.5,0.5);
	glTexCoord2f(0.0,1.0);
	glVertex3f(-0.5,0.5,0.5);

	glEnd();

}

void display() {
   glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	int largura=glutGet(GLUT_WINDOW_WIDTH);
	int altura=glutGet(GLUT_WINDOW_HEIGHT);
	gluPerspective(45.f,(float)largura/(float)altura,0.001,100.);
    glMatrixMode(GL_MODELVIEW);

   glClearColor(0.0, 0.0, 0.0, 0.0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   glPushMatrix();
   glLoadIdentity();

   gluLookAt(0.0,0.0,-4.0,0,0,1.0,0.0,1.0,0.0);

   glMultMatrixf( (GLfloat *) modelview_matrix );

 //  imprimirMatriz(modelview_matrix);


   DesenharCubo();

   glPopMatrix();

   glutSwapBuffers(); //Atualiza o BUFFER
}

float *readPPM(char *ppmfile, int *width, int *height)
{
	FILE *p=fopen(ppmfile,"r");
	if(p==NULL) return NULL;
	char *aux=new char[4];
	fscanf(p,"%s",aux);
	int h,w,aux2;
	fscanf(p,"%d %d",&w,&h);
	fscanf(p,"%d",&aux2);
	delete [] aux;

	float *img=new float[h*w*3];
	for (int r = 0; r < h; r++)
		for (int c = 0; c < w; c++)
		{
			int red,green,blue;
			fscanf(p,"%d %d %d",&red,&green,&blue);
			img[3*(r*w+c)]=(float)red/255.;
			img[3*(r*w+c)+1]=(float)green/255.;
			img[3*(r*w+c)+2]=(float)blue/255.;
		}

	*width=w;
	*height=h;
	return img;
}

/***
* init():
*   Configura o OpenGL.
***/
void init() {
   glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	int largura=glutGet(GLUT_WINDOW_WIDTH);
	int altura=glutGet(GLUT_WINDOW_HEIGHT);
	gluPerspective(45.f,(float)largura/(float)altura,0.001,100.);
    glMatrixMode(GL_MODELVIEW);
   	glLoadIdentity();
	glGetFloatv(GL_MODELVIEW_MATRIX,(GLfloat *) modelview_matrix);
	glEnable(GL_DEPTH_TEST);

	int w1,w2,h1,h2;

	float *image1=readPPM("textura1.ppm",&w1,&h1);
	float *image2=readPPM("textura2.ppm",&w2,&h2);

	if(image1!=NULL && image2!=NULL)
	{
		glGenTextures(2, texture);

		glBindTexture(GL_TEXTURE_2D, texture[0]);
		glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL );
		glTexImage2D(GL_TEXTURE_2D,0,3,w1,h1,0,GL_RGB,GL_FLOAT, image1);
		glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
		glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);

		glBindTexture(GL_TEXTURE_2D, texture[1]);
		glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL );
		glTexImage2D(GL_TEXTURE_2D,0,3,w2,h2,0,GL_RGB,GL_FLOAT, image2);
		glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
		glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	}
}

void push(int button, int state, int x, int y)
{

    ultimo_mouse=projetarNaEsfera(x,y);
}

void drag(int x, int y)
{
	printf("\n ln %f %f",ultimo_mouse.x,ultimo_mouse.y);


	vetor3D p1=projetarNaEsfera(x,y);
	//printf("\n%f %f %f",p1.x,p1.y,p1.z);
	vetor3D eixo_rot = vetorial(ultimo_mouse,p1);

	vetor3D p1_unit=versor(p1);
	vetor3D eixo_rot_unit=versor(eixo_rot);

	double angulo = acos(escalar(p1_unit,ultimo_mouse));


	glLoadIdentity();
	glRotatef(angulo*180.0/PI,eixo_rot_unit.x,eixo_rot_unit.y,eixo_rot_unit.z);
	glMultMatrixf( (GLfloat *) modelview_matrix );
	glGetFloatv(GL_MODELVIEW_MATRIX,(GLfloat *) modelview_matrix);

	ultimo_mouse.x=p1_unit.x;
	ultimo_mouse.y=p1_unit.y;
	ultimo_mouse.z=p1_unit.z;

	printf(" %f %f",ultimo_mouse.x,ultimo_mouse.y);

	glutPostRedisplay();
}


/***
* main():
*    Esta  a funo que  chamada quando executamos o programa. Vamos inicializar janelas
e configurar o GLUT aqui.
***/
int main(int argc, char** argv)
{
	modelview_matrix=new float[16];
   // Inicializa o ambiente do GLUT
   glutInit(&argc, argv);

   // Configuraes de janela e de sistema de cor.
   glutInitDisplayMode (GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
   glutInitWindowSize(250,250);
   glutInitWindowPosition(100,100);

   // Cria a janela do GLUT com o ttulo "Hello World!"
   glutCreateWindow("Hello World!");

   // Determina qual funo deve ser chamada para desenhar na tela (draw)
   glutDisplayFunc(display);

   // Determina qual funo deve ser chamada quando o usurio apertar o boto do mouse
   glutMouseFunc (push);
      // Determina qual funo deve ser chamada quando o usurio realiza DRAG
   glutMotionFunc(drag);

   // Esta funo inicializa configuraes do OpenGL
   init();

   // Exibe a janela e desenha na tela, usando a funo determinada em glutDisplayFunc
   glutMainLoop();
   return 0;
}